import oqupy
import numpy as np
import matplotlib.pyplot as plt


def gME(gd,gu,t):
    return gd * (1-np.exp(-t*(gd+gu))) / (gd+gu)

def SD(alpha,omega_cutoff,p,x):
    return 2*alpha*x**p*np.exp(-x/omega_cutoff)/omega_cutoff**(p-1)

def nb(T,x):
    if T!=0: return 1/(np.exp(x/T)-1)
    else: return 0

sigma_x = oqupy.operators.sigma("x")
sigma_y = oqupy.operators.sigma("y")
sigma_z = oqupy.operators.sigma("z")
sigma_p = (sigma_x + 1j * sigma_y)/2
sigma_m = (sigma_x - 1j * sigma_y)/2
up_density_matrix = oqupy.operators.spin_dm("z+")
down_density_matrix = oqupy.operators.spin_dm("z-")
I=oqupy.operators.identity(2)


p=3.0
epsilon = 0.5
Delta=1
D=0
# Delta=0.5
# D=0.2
mu=0.01
omega_cutoff = 1.0
alpha = 1/(2*np.pi)
T=0.5

tmax=20000


tks = [25,27,28]
dts = [0.1,.09,.08]
#tk=20
#dt=0.5




#J[x] = 2 * alpha * x^p / omega_cutoff^{p-1} * Exp[- x / omega_cutoff]

Reorg = 4 * alpha * omega_cutoff
h=Delta-D

chi_DeltaDelta = 2 * Reorg * h * Delta
chi_DD = 2 * Reorg * h * D
chi_mumu = 2 * Reorg * h * mu 

H_system = (epsilon + chi_DeltaDelta) * sigma_z + chi_mumu * sigma_x 
H_int = Delta * (sigma_z + I) + mu * sigma_x


# H_system = epsilon * sigma_z
# H_int = Delta * sigma_z + mu * sigma_x
for i in range(len(tks)):
	#for dt in dts:
		tk = tks[i]
		dt=dts[i]
		system = oqupy.System(H_system)
		correlations = oqupy.PowerLawSD(alpha=alpha,
				                zeta=p,
				                cutoff=omega_cutoff,
				                cutoff_type='exponential',
				                temperature = T)
		bath = oqupy.Bath(H_int, correlations)


		tempo_parameters = oqupy.TempoParameters(dt=dt, dkmax=tk/dt, epsrel=10**-7)

		dynamics = oqupy.tempo_compute(system=system,
				               bath=bath,
				               initial_state=down_density_matrix,
				               start_time=0.0,
				               end_time=tmax,
				               parameters=tempo_parameters)
		t, pgg = dynamics.expectations(down_density_matrix, real=True)
		t, pee = dynamics.expectations(up_density_matrix, real=True)
		t, sx = dynamics.expectations(sigma_x, real=True)
		t, sy = dynamics.expectations(sigma_y, real=True)


		plt.plot(t, pgg)
		# plt.plot(t, gAnalytic)
		plt.xlabel(r'$t$')
		plt.ylabel(r'$\rho_{gg}$')
		plt.legend()
		plt.savefig('Plot tk'+str(tk)+' dt'+str(dt)+'.png')
		np.savetxt('timeTEMPO_tk'+str(tk)+' dt'+str(dt)+'.csv', np.real(t),  delimiter=',')
		np.savetxt('pggTEMPO_'+str(tk)+' dt'+str(dt)+'.csv', np.real(pgg),  delimiter=',')
		np.savetxt('pegRTEMPO_'+str(tk)+' dt'+str(dt)+'.csv', np.real(0.5*sx),  delimiter=',')
		np.savetxt('pegITEMPO_'+str(tk)+' dt'+str(dt)+'.csv', np.real(0.5*sy),  delimiter=',')
		print('Complete cycle')
# fig, ax = plt.subplots(1,1)
# oqupy.helpers.plot_correlations_with_parameters(bath.correlations, tempo_parameters, ax=ax)}
